/*
 *	BIRD -- Label distribution protocol
 *
 *	(c) 2011 Alexander V. Chernikov <melifaro@ipfw.ru>
 *
 *	Can be freely distributed and used under the terms of the 
 *	GNU GPL or Simplified BSD License.
 */
#ifndef _BIRD_LDP_DATABASE_H_
#define _BIRD_LDP_DATABASE_H_

enum LDP_LIB_TYPE {
	LDP_LIB_TYPE_GENERAL,	/* Should not be used */
	LDP_LIB_TYPE_L3VPN,
	LDP_LIB_TYPE_L2P2P,	/* point2port L2 labels */
	LDP_LIB_TYPE_TRANSIT	/* Simple P-P transit (swap action)*/
};

#define LDP_LIB_FLAG_LOCAL	0x01 /* Record is locally-originated */
#define LDP_LIB_FLAG_TLV	0x02 /* One or more TLV is contained inside 'tlvs' list */

/* L3VPN, tunnels and other similar types */
typedef struct _ldp_lib_general {
	struct fib_node		n;
	struct bird_mpls_entry	me;
	uint16_t		type;
	uint16_t		flags;
	struct proto		*proto; /* pointer to protocol requesting label */
	service_relfunc		relfunc; /* pointer to label release function */
	list			tlvs;	/* Unknown TLVs with transit flag set */
	char			info[32]; /* text represenattion of requested item */
} ldp_lib_general;

/* labels for general IGP routes */
typedef struct _ldp_lib_transit {
	struct fib_node		n;
	struct bird_mpls_entry	me;
	uint16_t		type;
	uint16_t		flags;
	list			tlvs;
	ip_addr			prefix;
	int			pxlen;
	ip_addr			nexthop;
	ldp_session		*ls;
	node			n_lss; /* Per-session label list */
} ldp_lib_transit;
#if 0
/* labels for directly connected routes */
typedef struct _ldp_lib_direct {
	struct fib_node		n;
	struct bird_mpls_entry	me;
	uint16_t		type;
	uint16_t		flags;
	list			tlvs;
	ip_addr			prefix;
	int			pxlen;
} ldp_lib_direct;
#endif
/* MAXIMUM size of mpls_* data structure */
#define LDP_LIB_ENTRY_SIZE	sizeof(struct _ldp_lib_general)

typedef struct _ldp_lib_free_label {
	node			n;
	int			label;
} ldp_lib_free_label;

typedef struct _ldp_lib_database {
	struct fib		f;		/* fib to store label data */
	int			size;
	list			free_items;	/* List of items currently free */
	int			search_idx;
	int			current_label;	/* Current (used) label */
} ldp_lib_database;

/* RIB */
typedef struct _ldp_rib_group {
	struct fib_node		n;
	int			flags;
	list			entries;
} ldp_rib_group;
#define LDP_RG_NEW		0x01

typedef struct _ldp_rib_entry {
	node			n;	/* Per rib grouplist */
	uint32_t		label;	/* LOCAL MPLS label */
	struct bird_mpls_entry	me; /* REMOTE MPLS labelstack */
	uint16_t		type;	/* Node type */
	uint16_t		flags;
	ldp_session		*ls;
	ldp_rib_group		*rg;	/* Pointer to RIB group it belongs to */
	node			n_lss;	/* Per session RIB list */
	list			attrs;	/* List of transitive attributes */
} ldp_rib_entry;
#define LDP_RE_HASLABEL		0x01	/* entry has LOCAL label allocated in LFIB */
#define LDP_RE_BEST		0x02	/* entry is currently selected as best route */
#define LDP_RE_INSTALLED	0x04	/* entry hash MPLS label installed in kernel */

ldp_lib_general *ldp_lib_label_add(struct proto_ldp *pl, enum LDP_LIB_TYPE type, enum kmpls_op op);
int ldp_lib_label_del(struct proto_ldp *pl, u32 label);
ldp_lib_general *ldp_lib_label_find(struct proto_ldp *pl, u32 label);
char *ldp_lib_texttype(enum LDP_LIB_TYPE type);

ldp_rib_entry *ldp_rib_add_entry(struct proto_ldp *pl, ip_addr prefix, int plen, ldp_session *ls);
void ldp_rib_set_best(struct proto_ldp *pl, ldp_rib_entry *re);
int ldp_rib_del_entry(struct proto_ldp *pl, ldp_rib_entry *re);
ldp_rib_group *ldp_rib_search_group(struct proto_ldp *pl, ip_addr prefix, int plen);
ldp_rib_entry *ldp_rib_search_entry(struct proto_ldp *pl, ip_addr prefix, int plen, ldp_session *ls);
#if 0

ldp_rib_entry * ldp_rib_add_entry(struct proto_ldp *pl, ip_addr prefix, int plen, ldp_session *ls);
int ldp_rib_del_entry(struct proto_ldp *pl, ldp_rib_entry *re);
void ldp_rib_set_best(struct proto_ldp *pl, ldp_rib_entry *re);
void ldp_rib_fill_sockaddr(ldp_rib_entry *re, int label);
ldp_rib_group * ldp_rib_search_group(struct proto_ldp *pl, ip_addr prefix, int plen);
ldp_rib_entry * ldp_rib_search_entry(struct proto_ldp *pl, ip_addr prefix, int plen, ldp_session *ls);

ldp_lib_general * ldp_lib_label_add(struct proto_ldp *pl, enum LDP_LIB_TYPE type);
void ldp_lib_label_del(struct proto_ldp *pl, int label);
char * ldp_mplsop_texttype(enum kmpls_op op);
char * ldp_lib_texttype(enum LDP_LIB_TYPE type);

#endif
typedef struct _ldp_addrmap {
	node			n_hash;	/* Hash with all nexthop addresses */
	node			n_lss;	/* Per session list */
	ip_addr			addr;	/* LSR interface address */
	struct _ldp_session	*ls;	/* Pointer to session to which nexthop belongs to */
} ldp_addrmap;




#endif

